@@ -30,32 +30,44 @@ from nuitka.build import SconsInterface
 from nuitka.Options import assumeYesForDownloads, getIconPaths, getJobLimit
 from nuitka.OutputDirectories import getResultBasepath, getResultFullpath
 from nuitka.plugins.Plugins import Plugins
-from nuitka.PostProcessing import version_resources
+from nuitka.PostProcessing import (
+    executePostProcessingResources,
+    version_resources,
+)
 from nuitka.Tracing import general, postprocessing_logger, scons_logger
 from nuitka.utils.Download import getCachedDownload
-from nuitka.utils.Execution import getNullOutput, withEnvironmentVarsOverriden
+from nuitka.utils.Execution import (
+    getNullInput,
+    getNullOutput,
+    withEnvironmentVarsOverriden,
+)
 from nuitka.utils.FileOperations import (
     addFileExecutablePermission,
     getFileList,
     removeDirectory,
 )
-from nuitka.utils.Utils import getArchitecture, getOS
+from nuitka.utils.SharedLibraries import locateDLL
+from nuitka.utils.Utils import getArchitecture, getOS, hasOnefileSupportedOS
 
 
 def packDistFolderToOnefile(dist_dir, binary_filename):
     """Pack distribution to onefile, i.e. a single file that is directly executable."""
 
+    onefile_output_filename = getResultFullpath(onefile=True)
+
     if getOS() == "Linux":
-        packDistFolderToOnefileLinux(dist_dir, binary_filename)
+        packDistFolderToOnefileLinux(onefile_output_filename, dist_dir, binary_filename)
     elif getOS() == "Windows":
-        packDistFolderToOnefileWindows(dist_dir)
+        packDistFolderToOnefileWindows(onefile_output_filename, dist_dir)
     else:
         postprocessing_logger.sysexit(
             "Onefile mode is not yet available on %r." % getOS()
         )
 
+    Plugins.onOnefileFinished(onefile_output_filename)
 
-def getAppImageToolPath():
+
+def _getAppImageToolPath(for_operation, assume_yes_for_downloads):
     """Return the path of appimagetool (for Linux).
 
     Will prompt the user to download if not already cached in AppData
@@ -76,18 +88,27 @@ def getAppImageToolPath():
         message="""\
 Nuitka will make use of AppImage (https://appimage.org/) tool
 to combine Nuitka dist folder to onefile binary.""",
-        reject="Nuitka does not work in --onefile on Linux without.",
-        assume_yes_for_downloads=assumeYesForDownloads(),
+        reject="Nuitka does not work in --onefile on Linux without."
+        if for_operation
+        else None,
+        assume_yes_for_downloads=assume_yes_for_downloads,
     )
 
 
-def packDistFolderToOnefileLinux(dist_dir, binary_filename):
+def packDistFolderToOnefileLinux(onefile_output_filename, dist_dir, binary_filename):
     """Pack to onefile binary on Linux.
 
     Notes: This is mostly a wrapper around AppImage, which does all the heavy
     lifting.
     """
 
+    if not locateDLL("fuse"):
+        postprocessing_logger.sysexit(
+            """\
+Error, the fuse library (libfuse.so.x from fuse2, *not* fuse3) must be installed
+for onefile creation to work on Linux."""
+        )
+
     # This might be possible to avoid being done with --runtime-file.
     apprun_filename = os.path.join(dist_dir, "AppRun")
     with open(apprun_filename, "w") as output_file:
@@ -128,12 +149,12 @@ Categories=Utility;"""
         "Creating single file from dist folder, this may take a while."
     )
 
-    onefile_output_filename = getResultFullpath(onefile=True)
-
     # Starting the process while locked, so file handles are not duplicated.
     appimagetool_process = subprocess.Popen(
         (
-            getAppImageToolPath(),
+            _getAppImageToolPath(
+                for_operation=True, assume_yes_for_downloads=assumeYesForDownloads()
+            ),
             dist_dir,
             "--comp",
             "xz",
@@ -141,6 +162,7 @@ Categories=Utility;"""
             onefile_output_filename,
         ),
         shell=False,
+        stdin=getNullInput(),
         stderr=getNullOutput(),
         stdout=getNullOutput(),
     )
@@ -160,9 +182,6 @@ Categories=Utility;"""
 
 
 def _runOnefileScons(quiet):
-    # Scons gets transported many details, that we express as variables, and
-    # have checks for them, leading to many branches and statements,
-    # pylint: disable=too-many-branches,too-many-statements
 
     source_dir = OutputDirectories.getSourceDirectoryPath(onefile=True)
     SconsInterface.cleanSconsDirectory(source_dir)
@@ -183,54 +202,7 @@ def _runOnefileScons(quiet):
         "compiled_exe": OutputDirectories.getResultFullpath(onefile=False),
     }
 
-    # Ask Scons to cache on Windows, except where the directory is thrown
-    # away. On non-Windows you can should use ccache instead.
-    if not Options.isRemoveBuildDir() and getOS() == "Windows":
-        options["cache_mode"] = "true"
-
-    if Options.isLto():
-        options["lto_mode"] = "true"
-
-    if Options.shallDisableConsoleWindow():
-        options["win_disable_console"] = "true"
-
-    if Options.isShowScons():
-        options["show_scons"] = "true"
-
-    if Options.isMingw64():
-        options["mingw_mode"] = "true"
-
-    if Options.getMsvcVersion():
-        msvc_version = Options.getMsvcVersion()
-
-        msvc_version = msvc_version.replace("exp", "Exp")
-        if "." not in msvc_version:
-            msvc_version += ".0"
-
-        options["msvc_version"] = msvc_version
-
-    if getOS() == "Windows":
-        options["noelf_mode"] = "true"
-
-    if Options.isClang():
-        options["clang_mode"] = "true"
-
-    cpp_defines = Plugins.getPreprocessorSymbols()
-    if cpp_defines:
-        options["cpp_defines"] = ",".join(
-            "%s%s%s" % (key, "=" if value else "", value or "")
-            for key, value in cpp_defines.items()
-        )
-
-    link_libraries = Plugins.getExtraLinkLibraries()
-    if link_libraries:
-        options["link_libraries"] = ",".join(link_libraries)
-
-    if Options.shallRunInDebugger():
-        options["full_names"] = "true"
-
-    if Options.assumeYesForDownloads():
-        options["assume_yes_for_downloads"] = "true"
+    SconsInterface.setCommonOptions(options)
 
     onefile_env_values = {}
 
@@ -281,18 +253,18 @@ def _pickCompressor():
         return b"X", lambda data: data
 
 
-def packDistFolderToOnefileWindows(dist_dir):
+def packDistFolderToOnefileWindows(onefile_output_filename, dist_dir):
     postprocessing_logger.info(
         "Creating single file from dist folder, this may take a while."
     )
 
-    onefile_output_filename = getResultFullpath(onefile=True)
-
     general.info("Running bootstrap binary compilation via Scons.")
 
     # First need to create the bootstrap binary for unpacking.
     _runOnefileScons(quiet=not Options.isShowScons())
 
+    executePostProcessingResources(manifest=None, onefile=True)
+
     # Now need to append to payload it, potentially compressing it.
     compression_indicator, compressor = _pickCompressor()
 
@@ -327,3 +299,14 @@ def packDistFolderToOnefileWindows(dist_dir):
         output_file.write(b"\0\0")
 
         output_file.write(struct.pack("Q", start_pos))
+
+
+def checkOnefileReadiness(assume_yes_for_downloads):
+    if getOS() == "Linux":
+        app_image_path = _getAppImageToolPath(
+            for_operation=False, assume_yes_for_downloads=assume_yes_for_downloads
+        )
+
+        return app_image_path is not None
+    else:
+        return hasOnefileSupportedOS()
