@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -95,12 +95,12 @@ class PythonModuleBase(NodeBase):
                 module_name=package_name,
                 parent_package=None,
                 level=1,
-                warn=python_version < 300,
+                warn=python_version < 0x300,
             )
 
             # TODO: Temporary, if we can't find the package for Python3.3 that
             # is semi-OK, maybe.
-            if python_version >= 300 and not package_filename:
+            if python_version >= 0x300 and not package_filename:
                 return ()
 
             if package_name == "uniconvertor.app.modules":
@@ -217,11 +217,30 @@ class CompiledPythonModule(
 
     kind = "COMPILED_PYTHON_MODULE"
 
+    __slots__ = (
+        "is_top",
+        "name",
+        "code_prefix",
+        "code_name",
+        "uids",
+        "temp_variables",
+        "temp_scopes",
+        "preserver_id",
+        "needs_annotations_dict",
+        "trace_collection",
+        "mode",
+        "variables",
+        "active_functions",
+        "visited_functions",
+        "cross_used_functions",
+        "used_modules",
+        "future_spec",
+        "source_code",
+        "module_dict_name",
+        "locals_scope",
+    )
+
     named_children = ("body", "functions")
-    getBody = ChildrenHavingMixin.childGetter("body")
-    setBody = ChildrenHavingMixin.childSetter("body")
-    getFunctions = ChildrenHavingMixin.childGetter("functions")
-    setFunctions = ChildrenHavingMixin.childSetter("functions")
 
     checkers = {"body": checkStatementsSequenceOrNone}
 
@@ -297,6 +316,9 @@ class CompiledPythonModule(
     def getFutureSpec(self):
         return self.future_spec
 
+    def setFutureSpec(self, future_spec):
+        self.future_spec = future_spec
+
     def isTopModule(self):
         return self.is_top
 
@@ -425,10 +447,10 @@ class CompiledPythonModule(
         return encodePythonIdentifierToC(self.getFullName())
 
     def addFunction(self, function_body):
-        functions = self.getFunctions()
+        functions = self.subnode_functions
         assert function_body not in functions
         functions += (function_body,)
-        self.setFunctions(functions)
+        self.setChild("functions", functions)
 
     def startTraversal(self):
         self.used_modules = OrderedSet()
@@ -444,7 +466,7 @@ class CompiledPythonModule(
         return self.used_modules
 
     def addUsedFunction(self, function_body):
-        assert function_body in self.getFunctions(), function_body
+        assert function_body in self.subnode_functions, function_body
 
         assert (
             function_body.isExpressionFunctionBody()
@@ -465,7 +487,7 @@ class CompiledPythonModule(
         return self.active_functions
 
     def getUnusedFunctions(self):
-        for function in self.getFunctions():
+        for function in self.subnode_functions:
             if function not in self.active_functions:
                 yield function
 
@@ -477,7 +499,7 @@ class CompiledPythonModule(
         return self.cross_used_functions
 
     def getFunctionFromCodeName(self, code_name):
-        for function in self.getFunctions():
+        for function in self.subnode_functions:
             if function.getCodeName() == code_name:
                 return function
 
@@ -502,7 +524,7 @@ class CompiledPythonModule(
 
         self.trace_collection = TraceCollectionModule(self)
 
-        module_body = self.getBody()
+        module_body = self.subnode_body
 
         if module_body is not None:
             result = module_body.computeStatementsSequence(
@@ -510,7 +532,7 @@ class CompiledPythonModule(
             )
 
             if result is not module_body:
-                self.setBody(result)
+                self.setChild("body", result)
 
         new_modules = self.attemptRecursion()
 
@@ -521,10 +543,45 @@ class CompiledPythonModule(
                 message="Recursed to module package.",
             )
 
+        # Finalize locals scopes previously determined for removal in last pass.
         self.trace_collection.updateVariablesFromCollection(
             old_collection, self.source_ref
         )
 
+        # Indicate if this is pass 1 for the module as return value.
+        was_complete = not self.locals_scope.complete
+
+        def markAsComplete(body, trace_collection):
+            if (
+                body.locals_scope is not None
+                and body.locals_scope.isMarkedForPropagation()
+            ):
+                body.locals_scope = None
+
+            if body.locals_scope is not None:
+                body.locals_scope.markAsComplete(trace_collection)
+
+        def markEntryPointAsComplete(body):
+            markAsComplete(body, body.trace_collection)
+
+            outline_bodies = body.trace_collection.getOutlineFunctions()
+
+            if outline_bodies is not None:
+                for outline_body in outline_bodies:
+                    markAsComplete(outline_body, body.trace_collection)
+
+            body.optimizeUnusedTempVariables()
+
+        markEntryPointAsComplete(self)
+
+        for function_body in self.getUsedFunctions():
+            markEntryPointAsComplete(function_body)
+
+            function_body.optimizeUnusedClosureVariables()
+            function_body.optimizeVariableReleases()
+
+        return was_complete
+
     def getTraceCollections(self):
         yield self.trace_collection
 
@@ -688,7 +745,10 @@ class UncompiledPythonPackage(UncompiledPythonModule):
 class PythonMainModule(CompiledPythonModule):
     kind = "PYTHON_MAIN_MODULE"
 
+    __slots__ = ("main_added",)
+
     def __init__(self, main_added, mode, future_spec, source_ref):
+        # Is this one from a "__main__.py" file.
         self.main_added = main_added
 
         CompiledPythonModule.__init__(
